@@ -1,5 +1,5 @@
 import csv
-from typing import List, Generator, Any
+from typing import List, Generator, Any, Dict, IO
 
 import pandas as pd
 
@@ -7,9 +7,15 @@ from .dataset import Dataset
 from ..types import RecordType
 
 
-class CsvDataset(Dataset):
+class CsvDataset(Dataset[IO[str]]):
+    READ_ARGS: Dict[str, Any] = {
+        "mode": "r",
+        "encoding": "utf-8",
+        "errors": "ignore",
+    }
+
     def count_rows(self) -> int:
-        with open(self.filepath, "r", encoding="utf-8", errors="ignore") as f:
+        with self.fileobj() as f:
             reader = csv.reader(f)
 
             # handle case where CSV is empty
@@ -19,24 +25,25 @@ class CsvDataset(Dataset):
             )
 
     def get_columns(self) -> List[str]:
-        with open(self.filepath, "r", encoding="utf-8", errors="ignore") as f:
+        with self.fileobj() as f:
             reader = csv.reader(f)
             columns = next(reader)
             return columns
 
     def read_streaming_records(self) -> Generator[RecordType, None, None]:
-        with open(self.filepath, "r", encoding="utf-8", errors="ignore") as f:
+        with self.fileobj() as f:
             reader = csv.reader(f)
             columns = next(reader)
             for row in reader:
                 yield dict(zip(columns, row))
 
     def read_streaming_values(self) -> Generator[List[Any], None, None]:
-        with open(self.filepath, "r", encoding="utf-8", errors="ignore") as f:
+        with self.fileobj() as f:
             reader = csv.reader(f)
             next(reader)
             for row in reader:
                 yield row
 
     def read_file_as_dataframe(self) -> pd.DataFrame:
-        return pd.read_csv(self.filepath, keep_default_na=True)
+        with self.fileobj() as f:
+            return pd.read_csv(f, keep_default_na=True)
